@@ -7,16 +7,15 @@ use libc;
 use std::cell::{Cell, RefCell};
 use std::ptr;
 
-use drawing_ctx;
-use drawing_ctx::RsvgDrawingCtx;
-
-use aspect_ratio::*;
+use aspect_ratio::AspectRatio;
 use attributes::Attribute;
+use drawing_ctx::{self, RsvgDrawingCtx};
 use handle::RsvgHandle;
 use length::*;
 use node::*;
 use parsers::parse;
 use property_bag::PropertyBag;
+use state;
 
 struct NodeImage {
     aspect: Cell<AspectRatio>,
@@ -110,14 +109,8 @@ impl NodeTrait for NodeImage {
 
             let aspect = self.aspect.get();
 
-            if !drawing_ctx::state_is_overflow(state) {
-                if let Align::Aligned {
-                    fit: FitMode::Slice,
-                    ..
-                } = aspect.align
-                {
-                    drawing_ctx::add_clipping_rect(draw_ctx, x, y, w, h);
-                }
+            if !state::is_overflow(state) && aspect.is_slice() {
+                drawing_ctx::add_clipping_rect(draw_ctx, x, y, w, h);
             }
 
             let (x, y, w, h) = aspect.compute(
