@@ -112,12 +112,12 @@ fn _set_source_rsvg_solid_color(
     ctx: *mut drawing_ctx::RsvgDrawingCtx,
     color: &Color,
     opacity: u8,
-    current_color: u32,
+    current_color: Color,
 ) {
     let rgba_color = match *color {
         Color::RGBA(rgba) => Some(rgba),
         Color::CurrentColor => {
-            if let Color::RGBA(rgba) = Color::from(current_color) {
+            if let Color::RGBA(rgba) = current_color {
                 Some(rgba)
             } else {
                 None
@@ -213,18 +213,13 @@ pub extern "C" fn rsvg_paint_server_unref(paint_server: *const PaintServer) {
     unsafe { Rc::from_raw(paint_server) };
 }
 
-#[no_mangle]
-pub extern "C" fn _set_source_rsvg_paint_server(
+pub fn _set_source_rsvg_paint_server(
     c_ctx: *mut drawing_ctx::RsvgDrawingCtx,
-    c_ps: *const PaintServer,
+    ps: &PaintServer,
     opacity: u8,
-    c_bbox: RsvgBbox,
-    current_color: u32,
-) -> glib_sys::gboolean {
-    assert!(!c_ctx.is_null());
-    assert!(!c_ps.is_null());
-
-    let ps = unsafe { &*c_ps };
+    bbox: &RsvgBbox,
+    current_color: Color,
+) -> bool {
     let mut had_paint_server = false;
 
     match *ps {
@@ -232,23 +227,18 @@ pub extern "C" fn _set_source_rsvg_paint_server(
             ref iri,
             ref alternate,
         } => {
-            let node_ptr = drawing_ctx::acquire_node(c_ctx, iri.as_str());
-
-            if !node_ptr.is_null() {
-                let node = unsafe { &*node_ptr };
+            if let Some(acquired) = drawing_ctx::get_acquired_node(c_ctx, iri.as_str()) {
+                let node = acquired.get();
 
                 if node.get_type() == NodeType::LinearGradient
                     || node.get_type() == NodeType::RadialGradient
                 {
                     had_paint_server = gradient::gradient_resolve_fallbacks_and_set_pattern(
-                        node,
-                        c_ctx,
-                        opacity,
-                        &c_bbox,
+                        &node, c_ctx, opacity, bbox,
                     );
                 } else if node.get_type() == NodeType::Pattern {
                     had_paint_server =
-                        pattern::pattern_resolve_fallbacks_and_set_pattern(node, c_ctx, &c_bbox);
+                        pattern::pattern_resolve_fallbacks_and_set_pattern(&node, c_ctx, bbox);
                 }
             }
 
@@ -261,8 +251,6 @@ pub extern "C" fn _set_source_rsvg_paint_server(
                 );
                 had_paint_server = true;
             }
-
-            drawing_ctx::release_node(c_ctx, node_ptr);
         }
 
         PaintServer::SolidColor(color) => {
@@ -271,7 +259,23 @@ pub extern "C" fn _set_source_rsvg_paint_server(
         }
     };
 
-    had_paint_server.to_glib()
+    had_paint_server
+}
+
+#[no_mangle]
+pub extern "C" fn rsvg_set_source_rsvg_paint_server(
+    c_ctx: *mut drawing_ctx::RsvgDrawingCtx,
+    c_ps: *const PaintServer,
+    opacity: u8,
+    bbox: RsvgBbox,
+    current_color: u32,
+) -> glib_sys::gboolean {
+    assert!(!c_ctx.is_null());
+    assert!(!c_ps.is_null());
+
+    let ps = unsafe { &*c_ps };
+
+    _set_source_rsvg_paint_server(c_ctx, ps, opacity, &bbox, Color::from(current_color)).to_glib()
 }
 
 #[cfg(test)]
@@ -323,7 +327,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: None,
-            })
+            },)
         );
 
         assert_eq!(
@@ -331,7 +335,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: None,
-            })
+            },)
         );
 
         assert_eq!(
@@ -339,7 +343,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(Color::from(0xffff8040)),
-            })
+            },)
         );
 
         assert_eq!(
@@ -347,7 +351,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(Color::from(0x80ff8040)),
-            })
+            },)
         );
 
         assert_eq!(
@@ -355,7 +359,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: Some(Color::CurrentColor),
-            })
+            },)
         );
 
         assert_eq!(
@@ -363,7 +367,7 @@ mod tests {
             Ok(PaintServer::Iri {
                 iri: "#link".to_string(),
                 alternate: None,
-            })
+            },)
         );
     }
 
